@@ -13,6 +13,7 @@ class EspSerial(Serial):
 
     Attributes:
         esp: esptool.ESPLoader, will auto upload stub.
+        stub: esptool.ESPStubLoader, stubbed loader.
     """
 
     DEFAULT_BAUDRATE = 115200
@@ -33,6 +34,10 @@ class EspSerial(Serial):
             available_ports = esptool.get_port_list()
             ports = list(set(available_ports) - set(self.occupied_ports.keys()))
 
+            # sort to make /dev/ttyS* ports before /dev/ttyUSB* ports
+            # esptool will reverse the list
+            ports.sort()
+
             # prioritize the cache recorded target port
             if target:
                 for _port, _target in self._port_target_cache.items():
@@ -54,7 +59,7 @@ class EspSerial(Serial):
                 raise ValueError('Couldn\'t auto detect chip. Please manually specify with "--port"')
 
             # stub loader has more functionalities, need to run after calling `run_stub()`
-            self.stub: esptool.ESPLoader = None  # type: ignore
+            self.stub: esptool.ESPLoader = self.esp.run_stub()
 
             if baud > initial_baud:
                 self.esp.change_baud(baud)  # change back to the users settings
@@ -83,20 +88,17 @@ class EspSerial(Serial):
 
         @functools.wraps(func)
         def wrapper(self, *args, **kwargs):
-            self.proc.close()
-            self.proc.open()
-
-            settings = self.proc.get_settings()
-
-            try:
-                with DuplicateStdout(self.pexpect_proc):
-                    self.esp.connect('hard_reset')
-                    self.stub = self.esp.run_stub()
-                    ret = func(self, *args, **kwargs)
-                    self.stub.hard_reset()
-            finally:
-                self.proc.apply_settings(settings)
-                self.create_forward_io_thread(self.pexpect_proc)
+            with self.disable_redirect_thread() as killed:
+                settings = self.proc.get_settings()
+                try:
+                    with DuplicateStdout(self.pexpect_proc):
+                        if killed:
+                            self.esp.connect('hard_reset')
+                            self.stub = self.esp.run_stub()
+                        ret = func(self, *args, **kwargs)
+                        self.stub.hard_reset()
+                finally:
+                    self.proc.apply_settings(settings)
 
             return ret
 
